@@ -42,9 +42,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <gdk-pixbuf/gdk-pixbuf.h>
-#include <gdk/gdkkeysyms.h>
-
 #include "gtkbindings.h"
 #include "gtkframe.h"
 #include "gtkmain.h"
@@ -56,8 +53,9 @@
 #include "gtkstock.h"
 #include "gtkvbox.h"
 #include "gtkwindow.h"
-
+#include "gtktypebuiltins.h"
 #include "gtkintl.h"
+#include "a11y/gtkscalebuttonaccessible.h"
 
 /**
  * SECTION:gtkscalebutton
@@ -346,6 +344,8 @@ gtk_scale_button_class_init (GtkScaleButtonClass *klass)
 				"popup", 0);
   gtk_binding_entry_add_signal (binding_set, GDK_KEY_Escape, 0,
 				"popdown", 0);
+
+  gtk_widget_class_set_accessible_type (widget_class, GTK_TYPE_SCALE_BUTTON_ACCESSIBLE);
 }
 
 static void
@@ -562,8 +562,9 @@ gtk_scale_button_dispose (GObject *object)
  * @max: the maximum value of the scale (usually 100)
  * @step: the stepping of value when a scroll-wheel event,
  *        or up/down arrow event occurs (usually 2)
- * @icons: (allow-none): a %NULL-terminated array of icon names, or %NULL if
- *         you want to set the list later with gtk_scale_button_set_icons()
+ * @icons: (allow-none) (array zero-terminated=1): a %NULL-terminated
+ *         array of icon names, or %NULL if you want to set the list
+ *         later with gtk_scale_button_set_icons()
  *
  * Creates a #GtkScaleButton, with a range between @min and @max, with
  * a stepping of @step.
@@ -580,12 +581,12 @@ gtk_scale_button_new (GtkIconSize   size,
 		      const gchar **icons)
 {
   GtkScaleButton *button;
-  GtkAdjustment *adj;
+  GtkAdjustment *adjustment;
 
-  adj = gtk_adjustment_new (min, min, max, step, 10 * step, 0);
+  adjustment = gtk_adjustment_new (min, min, max, step, 10 * step, 0);
 
   button = g_object_new (GTK_TYPE_SCALE_BUTTON,
-                         "adjustment", adj,
+                         "adjustment", adjustment,
                          "icons", icons,
                          "size", size,
                          NULL);
@@ -643,7 +644,7 @@ gtk_scale_button_set_value (GtkScaleButton *button,
 /**
  * gtk_scale_button_set_icons:
  * @button: a #GtkScaleButton
- * @icons: a %NULL-terminated array of icon names
+ * @icons: (array zero-terminated=1): a %NULL-terminated array of icon names
  *
  * Sets the icons to be used by the scale button.
  * For details, see the #GtkScaleButton:icons property.
@@ -837,12 +838,12 @@ gtk_scale_button_scroll (GtkWidget      *widget,
 {
   GtkScaleButton *button;
   GtkScaleButtonPrivate *priv;
-  GtkAdjustment *adj;
+  GtkAdjustment *adjustment;
   gdouble d;
 
   button = GTK_SCALE_BUTTON (widget);
   priv = button->priv;
-  adj = priv->adjustment;
+  adjustment = priv->adjustment;
 
   if (event->type != GDK_SCROLL)
     return FALSE;
@@ -850,15 +851,15 @@ gtk_scale_button_scroll (GtkWidget      *widget,
   d = gtk_scale_button_get_value (button);
   if (event->direction == GDK_SCROLL_UP)
     {
-      d += adj->step_increment;
-      if (d > adj->upper)
-	d = adj->upper;
+      d += gtk_adjustment_get_step_increment (adjustment);
+      if (d > gtk_adjustment_get_upper (adjustment))
+	d = gtk_adjustment_get_upper (adjustment);
     }
   else
     {
-      d -= adj->step_increment;
-      if (d < adj->lower)
-	d = adj->lower;
+      d -= gtk_adjustment_get_step_increment (adjustment);
+      if (d < gtk_adjustment_get_lower (adjustment))
+	d = gtk_adjustment_get_lower (adjustment);
     }
   gtk_scale_button_set_value (button, d);
 
@@ -900,11 +901,10 @@ gtk_scale_popup (GtkWidget *widget,
   GtkAllocation allocation, dock_allocation, scale_allocation;
   GtkScaleButton *button;
   GtkScaleButtonPrivate *priv;
-  GtkAdjustment *adj;
+  GtkAdjustment *adjustment;
   gint x, y, m, dx, dy, sx, sy, startoff;
   gint min_slider_size;
   gdouble v;
-  GdkDisplay *display;
   GdkScreen *screen;
   gboolean is_moved;
   GdkDevice *device, *keyboard, *pointer;
@@ -912,9 +912,8 @@ gtk_scale_popup (GtkWidget *widget,
   is_moved = FALSE;
   button = GTK_SCALE_BUTTON (widget);
   priv = button->priv;
-  adj = priv->adjustment;
+  adjustment = priv->adjustment;
 
-  display = gtk_widget_get_display (widget);
   screen = gtk_widget_get_screen (widget);
   gtk_widget_get_allocation (widget, &allocation);
 
@@ -949,7 +948,7 @@ gtk_scale_popup (GtkWidget *widget,
   priv->timeout = TRUE;
 
   /* position (needs widget to be shown already) */
-  v = gtk_scale_button_get_value (button) / (adj->upper - adj->lower);
+  v = gtk_scale_button_get_value (button) / (gtk_adjustment_get_upper (adjustment) - gtk_adjustment_get_lower (adjustment));
   min_slider_size = gtk_range_get_min_slider_size (GTK_RANGE (priv->scale));
 
   if (priv->orientation == GTK_ORIENTATION_VERTICAL)
@@ -1027,7 +1026,7 @@ gtk_scale_popup (GtkWidget *widget,
 
   device = gdk_event_get_device (event);
 
-  if (device->source == GDK_SOURCE_KEYBOARD)
+  if (gdk_device_get_source (device) == GDK_SOURCE_KEYBOARD)
     {
       keyboard = device;
       pointer = gdk_device_get_associated_device (device);
@@ -1147,9 +1146,8 @@ gtk_scale_button_key_release (GtkWidget   *widget,
  * either the dock, or the scale itself */
 static void
 gtk_scale_button_grab_notify (GtkScaleButton *button,
-			      gboolean        was_grabbed)
+                              gboolean        was_grabbed)
 {
-  GdkDisplay *display;
   GtkScaleButtonPrivate *priv;
   GtkWidget *toplevel, *grab_widget;
   GtkWindowGroup *group;
@@ -1173,7 +1171,6 @@ gtk_scale_button_grab_notify (GtkScaleButton *button,
       gtk_widget_is_ancestor (grab_widget, priv->dock))
     return;
 
-  display = gtk_widget_get_display (priv->dock);
   gdk_device_ungrab (priv->grab_keyboard, GDK_CURRENT_TIME);
   gdk_device_ungrab (priv->grab_pointer, GDK_CURRENT_TIME);
   gtk_device_grab_remove (priv->dock, priv->grab_pointer);
@@ -1195,7 +1192,7 @@ cb_button_timeout (gpointer user_data)
 {
   GtkScaleButton *button;
   GtkScaleButtonPrivate *priv;
-  GtkAdjustment *adj;
+  GtkAdjustment *adjustment;
   gdouble val;
   gboolean res = TRUE;
 
@@ -1205,19 +1202,19 @@ cb_button_timeout (gpointer user_data)
   if (priv->click_id == 0)
     return FALSE;
 
-  adj = priv->adjustment;
+  adjustment = priv->adjustment;
 
   val = gtk_scale_button_get_value (button);
   val += priv->direction;
-  if (val <= adj->lower)
+  if (val <= gtk_adjustment_get_lower (adjustment))
     {
       res = FALSE;
-      val = adj->lower;
+      val = gtk_adjustment_get_lower (adjustment);
     }
-  else if (val > adj->upper)
+  else if (val > gtk_adjustment_get_upper (adjustment))
     {
       res = FALSE;
-      val = adj->upper;
+      val = gtk_adjustment_get_upper (adjustment);
     }
   gtk_scale_button_set_value (button, val);
 
@@ -1237,19 +1234,19 @@ cb_button_press (GtkWidget      *widget,
 {
   GtkScaleButton *button;
   GtkScaleButtonPrivate *priv;
-  GtkAdjustment *adj;
+  GtkAdjustment *adjustment;
 
   button = GTK_SCALE_BUTTON (user_data);
   priv = button->priv;
-  adj = priv->adjustment;
+  adjustment = priv->adjustment;
 
   if (priv->click_id != 0)
     g_source_remove (priv->click_id);
 
   if (widget == priv->plus_button)
-    priv->direction = fabs (adj->page_increment);
+    priv->direction = fabs (gtk_adjustment_get_page_increment (adjustment));
   else
-    priv->direction = - fabs (adj->page_increment);
+    priv->direction = - fabs (gtk_adjustment_get_page_increment (adjustment));
 
   priv->click_id = gdk_threads_add_timeout (priv->click_timeout,
                                             cb_button_timeout,
@@ -1301,22 +1298,18 @@ cb_dock_grab_broken_event (GtkWidget *widget,
   return FALSE;
 }
 
-/*
- * Scale callbacks.
- */
+/* Scale callbacks  */
 
 static void
 gtk_scale_button_release_grab (GtkScaleButton *button,
-			       GdkEventButton *event)
+                               GdkEventButton *event)
 {
   GdkEventButton *e;
-  GdkDisplay *display;
   GtkScaleButtonPrivate *priv;
 
   priv = button->priv;
 
   /* ungrab focus */
-  display = gtk_widget_get_display (GTK_WIDGET (button));
   gdk_device_ungrab (priv->grab_keyboard, event->time);
   gdk_device_ungrab (priv->grab_pointer, event->time);
   gtk_device_grab_remove (priv->dock, priv->grab_pointer);
@@ -1357,13 +1350,11 @@ gtk_scale_button_popdown (GtkWidget *widget)
 {
   GtkScaleButton *button;
   GtkScaleButtonPrivate *priv;
-  GdkDisplay *display;
 
   button = GTK_SCALE_BUTTON (widget);
   priv = button->priv;
 
   /* ungrab focus */
-  display = gtk_widget_get_display (widget);
   gdk_device_ungrab (priv->grab_keyboard, GDK_CURRENT_TIME);
   gdk_device_ungrab (priv->grab_pointer, GDK_CURRENT_TIME);
   gtk_device_grab_remove (priv->dock, priv->grab_pointer);
@@ -1532,8 +1523,7 @@ static void
 gtk_scale_button_update_icon (GtkScaleButton *button)
 {
   GtkScaleButtonPrivate *priv;
-  GtkRange *range;
-  GtkAdjustment *adj;
+  GtkAdjustment *adjustment;
   gdouble value;
   const gchar *name;
   guint num_icons;
@@ -1543,8 +1533,8 @@ gtk_scale_button_update_icon (GtkScaleButton *button)
   if (!priv->icon_list || priv->icon_list[0] == '\0')
     {
       gtk_image_set_from_stock (GTK_IMAGE (priv->image),
-				GTK_STOCK_MISSING_IMAGE,
-				priv->size);
+                                GTK_STOCK_MISSING_IMAGE,
+                                priv->size);
       return;
     }
 
@@ -1554,37 +1544,37 @@ gtk_scale_button_update_icon (GtkScaleButton *button)
   if (num_icons == 1)
     {
       gtk_image_set_from_icon_name (GTK_IMAGE (priv->image),
-				    priv->icon_list[0],
-				    priv->size);
+                                    priv->icon_list[0],
+                                    priv->size);
       return;
     }
 
-  range = GTK_RANGE (priv->scale);
-  adj = priv->adjustment;
+  adjustment = priv->adjustment;
   value = gtk_scale_button_get_value (button);
 
   /* The 2-icons special case */
   if (num_icons == 2)
     {
       gdouble limit;
-      limit = (adj->upper - adj->lower) / 2 + adj->lower;
+
+      limit = (gtk_adjustment_get_upper (adjustment) - gtk_adjustment_get_lower (adjustment)) / 2 + gtk_adjustment_get_lower (adjustment);
       if (value < limit)
-	name = priv->icon_list[0];
+        name = priv->icon_list[0];
       else
-	name = priv->icon_list[1];
+        name = priv->icon_list[1];
 
       gtk_image_set_from_icon_name (GTK_IMAGE (priv->image),
-				    name,
-				    priv->size);
+                                    name,
+                                    priv->size);
       return;
     }
 
   /* With 3 or more icons */
-  if (value == adj->lower)
+  if (value == gtk_adjustment_get_lower (adjustment))
     {
       name = priv->icon_list[0];
     }
-  else if (value == adj->upper)
+  else if (value == gtk_adjustment_get_upper (adjustment))
     {
       name = priv->icon_list[1];
     }
@@ -1593,15 +1583,15 @@ gtk_scale_button_update_icon (GtkScaleButton *button)
       gdouble step;
       guint i;
 
-      step = (adj->upper - adj->lower) / (num_icons - 2);
-      i = (guint) ((value - adj->lower) / step) + 2;
+      step = (gtk_adjustment_get_upper (adjustment) - gtk_adjustment_get_lower (adjustment)) / (num_icons - 2);
+      i = (guint) ((value - gtk_adjustment_get_lower (adjustment)) / step) + 2;
       g_assert (i < num_icons);
       name = priv->icon_list[i];
     }
 
   gtk_image_set_from_icon_name (GTK_IMAGE (priv->image),
-				name,
-				priv->size);
+                                name,
+                                priv->size);
 }
 
 static void
